/*=======================================================================
Projet: PPE Robot suiveur de ligne
Nom du fichier: MrLineTiny.c
Rle: Source de la librairie MrLineTiny.lib
Cr le : 25/12/2010
Dernire rvision le :
Auteur: Philippe MARIANO
--------------------------------------------------------------
Cross compilateur utilis: CodeVisionAVR V2.04.08a
Composant cible : Atiny26 (Robot MrLineTiny)
--------------------------------------------------------------
Cible Hardware  : Robot MrLineTiny
--------------------------------------------------------------
Remarque: Une fois compil par -> file --> Convert library
le fichier MrLineTiny.lib est automatiquement plac dans le rpertoire
CVAVR2\LIB
========================================================================*/


//*****************************************************************************************************
// Bibliothques
//-----------------------------------------------------------------------------------------------------
// CVAVR
//-----------------------------------------------------------------------------------------------------
#include <tiny26.h>
#include <delay.h>
//-----------------------------------------------------------------------------------------------------
// Dveloppes pour l'application
//-----------------------------------------------------------------------------------------------------
#include "MrLineTiny.h"
#include "RBELEVE.h"
// ****************************************************************************************************

// ****************************************************************************************************
// 				Fonctions ddies  la commande du robot MrLineTiny
//=====================================================================================================
// Rcapitulatif
// 1) Arret_Moteurs
// 2) Active_Timer
// 3) AcquerirPositionRobot
// 4) AfficherPositionRobot
// 5) ReglerCdeMoteursRobot
//=====================================================================================================
#if funcused Arret_Moteurs
void Arret_Moteurs()
{
      TCCR1A = 0x00; // Arrt PWM 
      TCCR1B = 0x00; 
      PORTB=0x0B;    // Arrt Moteur 
} 
#endif
//=====================================================================================================
#if funcused Active_Timer
void Active_Timer()
{                   // Marche PWM
    TCCR1A=0xA3;
    TCCR1B=0x03;
}
#endif
//=====================================================================================================
/* FS11 "Calculer la position du robot par rapport  la ligne"   
//-----------------------------------------------------------------------------------------------------
Rle : Etablir une valeur numrique reprsentative de la position du robot par rapport  la ligne
 
Entre(s):          ACO                     bit associ  la sortie du comparateur analogique, 
                                            reprsentatif de la position d'un capteur sur la ligne 
                    
Sortie(s):          Position_Robot          octet image de la position du robot par rapport  la ligne)
                                            (dtails ci-dessous)
                                        
Position des capteurs IR  =>  "Position_Robot"    // Positions du robot par rapport  la ligne
//-----------------------------------------------------------------------------------------------------
IR_G | IR_C  |  IR_D     =>     Position_Robot(10)
non  |  non  |   non     =>       0    // Hors ligne
non  |  non  |   oui     =>       1    // Gauche+
non  |  oui  |   non     =>       2    // Centre1 (ligne noire de largeur gale  celle d'un capteur)
non  |  oui  |   oui     =>       3    // Gauche
oui  |  non  |   non     =>       4    // Droite+
oui  |  non  |   oui     =>       5    // Centre2 (ligne blanche entoure de deux noires)
oui  |  oui  |   non     =>       6    // Droite
oui  |  oui  |   oui     =>       7    // Centre3 (ligne noire de largeur gale  celle de trois capteurs) 

                    Cde_Del_IR              octet permettant de multiplexer la commande des LED IR 
                                            Cde_Del_IR est recoupie dans le PORT A. Celui-ci commande 
                                            les LED IR (dtails ci-dessous)
                                            
                                            Emmetteur IR connect  au bit      Remarque 
                                            ----------------------------------------------------         
                                            E_D_IR        "      PA0    |       Emmeteur IR droit       
                                            E_C_IR        "      PA1    |       Emmeteur IR centre      
                                            E_G_IR        "      PA2    |       Emmeteur IR gauche                                                                   
-----------------------------------------------------------------------------------------------------*/
#if funcused AcquerirPositionRobot
unsigned char AcquerirPositionRobot(void)
{
    unsigned char i = 0;                    // Compteur de boucles
    unsigned char PositionCapteur =  0;     // Initialisation    
    unsigned char Cde_Del_IR = 1;           // Permet de multiplexer la commande des LED IR
          
      for (i=0;i<3;i++)  
       {
        PORTA = ~Cde_Del_IR;        // Les E_IR (LED IR) sont commands  l'tat bas
        delay_us(t_IR);             // Dure de l'tat bas (A ajuster en fonction des capteurs)
                                    // Si le capteur IR_x x=[D,C,G] est sur la ligne alors
                                    // le bit correspondant de "Position_Robot est mis  1
        if (ACO == 1) PositionCapteur = PositionCapteur | Cde_Del_IR;  // Calcul de la position du robot 
        PORTA = 0x3F;                                                 
        Cde_Del_IR = Cde_Del_IR << 1 ;                                 
        delay_us(t_mux);			// Tempo permettant d'attendre l'anulation de la ddp sur		      
       } 							// l'entre AIN1 (signal de rception IR) avant une nouvelle 
                                    // mesure (A ajuster en fonction des capteurs)
       return PositionCapteur;   
}
#endif   
//===================================================================================================== 
/* FS13 : " Choisir la valeur  afficher "
//-----------------------------------------------------------------------------------------------------
Rle :   Commande la LED correspondant au capteur sur la ligne

Entre(s):      Position_Robot      	(voir ci-dessus)

Sortie(s):	    VisuPosition            Octet non sign reprsentatif  de la position du robot 
                                        par rapport  la ligne
 					                    Led de visualisation connecte au bit
					                    ----------------------------------------
					                                Del_D       "      		PA3      
					                                Del_C       "	  		PA4
					                                Del_G       "	  		PA5 
					                    Remarque : D pour droite, C pour centre, G pour gauche
-----------------------------------------------------------------------------------------------------*/
#if funcused AfficherPositionRobot
unsigned char AfficherPositionRobot(unsigned char Position_Robot)
{
      unsigned char VisuPosition = 0;   
      
      switch (Position_Robot)              	       		    // Etat des Dels de visualisation     	
       {                                   	       		    // Del_G | Del_C | Del_D 	
        case 0: VisuPosition = Visu_Hors_ligne; break; 		// off   | off   | off 	
        case 1: VisuPosition = Visu_Gauche_Plus; break; 	// off   | off   | on 	
        case 2: VisuPosition = Visu_Centre1; break; 		// off   | on    | off 	
        case 3: VisuPosition = Visu_Gauche; break;     		// off   | on    | on 	
        case 4: VisuPosition = Visu_Droite_Plus; break; 	// on    | off   | off  		
        case 5: VisuPosition = Visu_Centre2; break; 		// on    | off   | on 	
        case 6: VisuPosition = Visu_Droite; break; 		    // on    | on    | off  	
        case 7: VisuPosition = Visu_Centre3; break;      	// on    | on    | on  	
       }
        
      return VisuPosition;
}
#endif   
//=====================================================================================================
/*-----------------------------------------------------------------------------------------------------
FS12 : "Choisir les valeurs des rapports cycliques"
//-----------------------------------------------------------------------------------------------------
Rle: Rgler le rapport cyclique des signaux de commande des moteurs droit et gauche pour corriger la 
trajectoire du robot afin qu'il suive la ligne.

Entre(s):      Position_Robot       (voir ci-dessus)    

Sortie(s):      CDE_ROUE_G           octet	    (OCR1A) rgle le rapport cyclique du signal de commande  
					                            du moteur associ  la roue Gauche
	            CDE_ROUE_D           octet      (OCR1B) rgle le rapport cyclique du signal de commande  
	                                            du moteur associ  la roue Droite
-----------------------------------------------------------------------------------------------------*/ 
#if funcused ReglerCdeMoteursRobot 
void ReglerCdeMoteursRobot (unsigned char Position_Robot)
{

static enum Etat_Commande_Moteur EtatCdeMot = DEPART; 
     
      switch (EtatCdeMot)              
       { // Dbut switch (EtatCdeMot)                                 
        case DEPART: Arret_Moteurs(); 
        		if ((Position_Robot == CENTRE_1) || (Position_Robot == CENTRE_2) || (Position_Robot == CENTRE_3))
        		    { 
        		      delay_ms(3000);
        		      Active_Timer();
        		      EtatCdeMot = CDE_CENTRE1;
        		    }
        		    else  EtatCdeMot = DEPART;
        break;   
        
        case CDE_CENTRE1: EtatCdeMot = CDE_CENTRE3; 
        break;  
        
        case CDE_CENTRE2: EtatCdeMot = CDE_CENTRE3;   
        
        break;   
        
        case CDE_CENTRE3: CDE_ROUE_G = _80PC; CDE_ROUE_D = _80PC;
                          if (Position_Robot == GAUCHE) EtatCdeMot = CDE_DROITE;
                          	else if (Position_Robot == DROITE) EtatCdeMot = CDE_GAUCHE;
                          		else EtatCdeMot = CDE_CENTRE3;
        break; 
        
        case CDE_DROITE:CDE_ROUE_G = _70PC; CDE_ROUE_D = _50PC; 
                         if (Position_Robot == CENTRE_3) EtatCdeMot = CDE_CENTRE3;
                          	else if (Position_Robot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS;
                          		else EtatCdeMot = CDE_DROITE;
        break;   
        
        case CDE_DROITE_PLUS:CDE_ROUE_G = _70PC; CDE_ROUE_D = _40PC;
        	if (Position_Robot == GAUCHE) EtatCdeMot = CDE_DROITE; 
        		else if (Position_Robot == HORS_LIGNE) EtatCdeMot = HORS_LIGNE_DROITE;
                          		else EtatCdeMot = CDE_DROITE_PLUS;
        break;  
        
        case HORS_LIGNE_DROITE:CDE_ROUE_G = _50PC; CDE_ROUE_D = _20PC;
        	if (Position_Robot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS; 
                          		else EtatCdeMot = HORS_LIGNE_DROITE;
        break; 
        
        case CDE_GAUCHE: CDE_ROUE_G = _50PC; CDE_ROUE_D = _70PC; 
                       if (Position_Robot == CENTRE_3) EtatCdeMot = CDE_CENTRE3;
                          	else if (Position_Robot == DROITE_PLUS) EtatCdeMot = CDE_GAUCHE_PLUS;
                          		else EtatCdeMot = CDE_GAUCHE;
        break;  
        
        case CDE_GAUCHE_PLUS: CDE_ROUE_G = _40PC; CDE_ROUE_D = _70PC; 
                       if (Position_Robot == DROITE) EtatCdeMot = CDE_GAUCHE;
                       		else if (Position_Robot == HORS_LIGNE) EtatCdeMot = HORS_LIGNE_GAUCHE;
                          		else EtatCdeMot = CDE_GAUCHE_PLUS;
        break; 
        
        case HORS_LIGNE_GAUCHE:CDE_ROUE_G = _20PC; CDE_ROUE_D = _70PC;
        	if (Position_Robot == DROITE_PLUS) EtatCdeMot = CDE_GAUCHE_PLUS; 
                          		else EtatCdeMot = HORS_LIGNE_GAUCHE;   
        break;
        
                          		
        default : EtatCdeMot = CDE_CENTRE3;
         
       }// Fin switch (EtatCdeMot) 
}
#endif       
//=====================================================================================================